@@ -8,7 +8,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkGrid.c,v 1.4 1999/04/16 01:51:14 stanton Exp $
+ * RCS: @(#) $Id: tkGrid.c,v 1.9 2000/04/17 17:40:27 ericm Exp $
  */
 
 #include "tkInt.h"
@@ -435,7 +435,7 @@ Tk_GridCmd(clientData, interp, argc, argv)
 		    slavePtr->iPadX = slavePtr->iPadY = 0;
 		    slavePtr->doubleBw = 2*Tk_Changes(tkwin)->border_width;
 		    if (slavePtr->flags & REQUESTED_RELAYOUT) {
-			Tk_CancelIdleCall(ArrangeGrid, (ClientData) slavePtr);
+			Tcl_CancelIdleCall(ArrangeGrid, (ClientData) slavePtr);
 		    }
 		    slavePtr->flags = 0;
 		    slavePtr->sticky = 0;
@@ -522,7 +522,7 @@ Tk_GridCmd(clientData, interp, argc, argv)
 	 */
 
 	while (masterPtr->flags & REQUESTED_RELAYOUT) {
-	    Tk_CancelIdleCall(ArrangeGrid, (ClientData) masterPtr);
+	    Tcl_CancelIdleCall(ArrangeGrid, (ClientData) masterPtr);
 	    ArrangeGrid ((ClientData) masterPtr);
 	}
 	SetGridSize(masterPtr);
@@ -575,8 +575,15 @@ Tk_GridCmd(clientData, interp, argc, argv)
 	if (Tcl_GetBoolean(interp, argv[3], &propagate) != TCL_OK) {
 	    return TCL_ERROR;
 	}
+	
+	/* Only request a relayout if the propagation bit changes */
+	
 	if ((!propagate) ^ (masterPtr->flags&DONT_PROPAGATE)) {
-	    masterPtr->flags  ^= DONT_PROPAGATE;
+	    if (propagate) {
+		masterPtr->flags &= ~DONT_PROPAGATE;
+	    } else {
+		masterPtr->flags |= DONT_PROPAGATE;
+	    }
       
 	    /*
 	     * Re-arrange the master to allow new geometry information to
@@ -1431,7 +1438,7 @@ ResolveConstraints(masterPtr, slotType, maxOffset)
 
     gridCount = MAX(constraintCount,slotCount);
     if (gridCount >= TYPICAL_SIZE) {
-	layoutPtr = (GridLayout *) Tcl_Alloc(sizeof(GridLayout) * (1+gridCount));
+	layoutPtr = (GridLayout *) ckalloc(sizeof(GridLayout) * (1+gridCount));
     } else {
 	layoutPtr = layoutData;
     }
@@ -1750,7 +1757,7 @@ GetGrid(tkwin)
     if (!new) {
 	return (Gridder *) Tcl_GetHashValue(hPtr);
     }
-    gridPtr = (Gridder *) Tcl_Alloc(sizeof(Gridder));
+    gridPtr = (Gridder *) ckalloc(sizeof(Gridder));
     gridPtr->tkwin = tkwin;
     gridPtr->masterPtr = NULL;
     gridPtr->masterDataPtr = NULL;
@@ -1873,7 +1880,7 @@ CheckSlotData(masterPtr, slot, slotType, checkOnly)
 	    int      newNumSlot = slot + PREALLOC ;
 	    size_t   oldSize = numSlot    * sizeof(SlotInfo) ;
 	    size_t   newSize = newNumSlot * sizeof(SlotInfo) ;
-	    SlotInfo *new = (SlotInfo *) Tcl_Alloc(newSize);
+	    SlotInfo *new = (SlotInfo *) ckalloc(newSize);
 	    SlotInfo *old = (slotType == ROW) ?
 		    masterPtr->masterDataPtr->rowPtr :
 		    masterPtr->masterDataPtr->columnPtr;
@@ -1925,19 +1932,19 @@ InitMasterData(masterPtr)
     size_t size;
     if (masterPtr->masterDataPtr == NULL) {
 	GridMaster *gridPtr = masterPtr->masterDataPtr =
-		(GridMaster *) Tcl_Alloc(sizeof(GridMaster));
+		(GridMaster *) ckalloc(sizeof(GridMaster));
 	size = sizeof(SlotInfo) * TYPICAL_SIZE;
 
 	gridPtr->columnEnd = 0;
 	gridPtr->columnMax = 0;
-	gridPtr->columnPtr = (SlotInfo *) Tcl_Alloc(size);
-	gridPtr->columnSpace = 0;
+	gridPtr->columnPtr = (SlotInfo *) ckalloc(size);
 	gridPtr->columnSpace = TYPICAL_SIZE;
 	gridPtr->rowEnd = 0;
 	gridPtr->rowMax = 0;
-	gridPtr->rowPtr = (SlotInfo *) Tcl_Alloc(size);
-	gridPtr->rowSpace = 0;
+	gridPtr->rowPtr = (SlotInfo *) ckalloc(size);
 	gridPtr->rowSpace = TYPICAL_SIZE;
+	gridPtr->startX = 0;
+	gridPtr->startY = 0;
 
 	memset((VOID *) gridPtr->columnPtr, 0, size);
 	memset((VOID *) gridPtr->rowPtr, 0, size);
@@ -2007,7 +2014,7 @@ Unlink(slavePtr)
  *
  * DestroyGrid --
  *
- *	This procedure is invoked by Tk_EventuallyFree or Tcl_Release
+ *	This procedure is invoked by Tcl_EventuallyFree or Tcl_Release
  *	to clean up the internal structure of a grid at a safe time
  *	(when no-one is using it anymore).   Cleaning up the grid involves
  *	freeing the main structure for all windows. and the master structure
@@ -2097,10 +2104,10 @@ GridStructureProc(clientData, eventPtr)
 	Tcl_DeleteHashEntry(Tcl_FindHashEntry(&dispPtr->gridHashTable,
 		(char *) gridPtr->tkwin));
 	if (gridPtr->flags & REQUESTED_RELAYOUT) {
-	    Tk_CancelIdleCall(ArrangeGrid, (ClientData) gridPtr);
+	    Tcl_CancelIdleCall(ArrangeGrid, (ClientData) gridPtr);
 	}
 	gridPtr->tkwin = NULL;
-	Tk_EventuallyFree((ClientData) gridPtr, DestroyGrid);
+	Tcl_EventuallyFree((ClientData) gridPtr, DestroyGrid);
     } else if (eventPtr->type == MapNotify) {
 	if (!(gridPtr->flags & REQUESTED_RELAYOUT)) {
 	    gridPtr->flags |= REQUESTED_RELAYOUT;
@@ -2505,6 +2512,7 @@ ConfigureSlaves(interp, tkwin, argc, argv)
 	    return TCL_ERROR;
 	}
 
+	/* Count the number of consecutive ^'s starting from this position */
 	for (width=1; width+j < numWindows && *argv[j+width] == REL_VERT;
 		width++) {
 	    /* Null Body */
@@ -2517,7 +2525,7 @@ ConfigureSlaves(interp, tkwin, argc, argv)
 	if (lastWindow == NULL) { 
 	    if (masterPtr->masterDataPtr != NULL) {
 		SetGridSize(masterPtr);
-		lastRow = masterPtr->masterDataPtr->rowEnd - 1;
+		lastRow = masterPtr->masterDataPtr->rowEnd - 2;
 	    } else {
 		lastRow = 0;
 	    }
@@ -2525,27 +2533,30 @@ ConfigureSlaves(interp, tkwin, argc, argv)
 	} else {
 	    other = Tk_NameToWindow(interp, lastWindow, tkwin);
 	    otherPtr = GetGrid(other);
-	    lastRow = otherPtr->row;
+	    lastRow = otherPtr->row + otherPtr->numRows - 2;
 	    lastColumn = otherPtr->column + otherPtr->numCols;
 	}
 
 	for (match=0, slavePtr = masterPtr->slavePtr; slavePtr != NULL;
 					 slavePtr = slavePtr->nextPtr) {
 
-	    if (slavePtr->numCols == width
-		    && slavePtr->column == lastColumn
-		    && slavePtr->row + slavePtr->numRows == lastRow) {
-		slavePtr->numRows++;
-		match++;
+	    if (slavePtr->column == lastColumn
+		    && slavePtr->row + slavePtr->numRows - 1 == lastRow) {
+		if (slavePtr->numCols <= width) {
+		    slavePtr->numRows++;
+		    match++;
+		    j += slavePtr->numCols - 1;
+		    lastWindow = Tk_PathName(slavePtr->tkwin);
+		    break;
+		}
 	    }
-	    lastWindow = Tk_PathName(slavePtr->tkwin);
 	}
 	if (!match) {
 	    Tcl_AppendResult(interp, "can't find slave to extend with \"^\".",
 		    (char *) NULL);
 	    return TCL_ERROR;
 	}
-	j += width - 1;
+/*	j += width - 1; */
     }
 
     if (masterPtr == NULL) {
