@@ -6,12 +6,12 @@
  *	one application can use as its main window an internal window from
  *	another application).
  *
- * Copyright (c) 1996 Sun Microsystems, Inc.
+ * Copyright (c) 1996-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkWinEmbed.c 1.20 97/11/05 17:47:09;
+ * RCS: @(#) $Id: tkWinEmbed.c,v 1.3.12.1 2001/04/04 07:57:18 hobbs Exp $
  */
 
 #include "tkWinInt.h"
@@ -38,9 +38,11 @@ typedef struct Container {
 					 * this process. */
 } Container;
 
-static Container *firstContainerPtr = NULL;
-					/* First in list of all containers
+typedef struct ThreadSpecificData {
+    Container *firstContainerPtr;       /* First in list of all containers
 					 * managed by this process.  */
+} ThreadSpecificData;
+static Tcl_ThreadDataKey dataKey;
 
 static void		CleanupContainerList _ANSI_ARGS_((
     			    ClientData clientData));
@@ -74,14 +76,16 @@ CleanupContainerList(clientData)
     ClientData clientData;
 {
     Container *nextPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
     
     for (;
-        firstContainerPtr != (Container *) NULL;
-        firstContainerPtr = nextPtr) {
-        nextPtr = firstContainerPtr->nextPtr;
-        ckfree((char *) firstContainerPtr);
+        tsdPtr->firstContainerPtr != (Container *) NULL;
+        tsdPtr->firstContainerPtr = nextPtr) {
+        nextPtr = tsdPtr->firstContainerPtr->nextPtr;
+        ckfree((char *) tsdPtr->firstContainerPtr);
     }
-    firstContainerPtr = (Container *) NULL;
+    tsdPtr->firstContainerPtr = (Container *) NULL;
 }
 
 /*
@@ -126,7 +130,7 @@ TkpTestembedCmd(clientData, interp, argc, argv)
  *	The return value is normally TCL_OK. If an error occurred (such as
  *	if the argument does not identify a legal Windows window handle),
  *	the return value is TCL_ERROR and an error message is left in the
- *	interp->result if interp is not NULL.
+ *	the interp's result if interp is not NULL.
  *
  * Side effects:
  *	None.
@@ -147,6 +151,8 @@ TkpUseWindow(interp, tkwin, string)
     int id;
     HWND hwnd;
     Container *containerPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     if (winPtr->window != None) {
         panic("TkpUseWindow: Already assigned a window");
@@ -159,7 +165,8 @@ TkpUseWindow(interp, tkwin, string)
 
     /*
      * Check if the window is a valid handle. If it is invalid, return
-     * TCL_ERROR and potentially leave an error message in interp->result.
+     * TCL_ERROR and potentially leave an error message in the interp's
+     * result.
      */
 
     if (!IsWindow(hwnd)) {
@@ -190,7 +197,7 @@ TkpUseWindow(interp, tkwin, string)
      * things will get cleaned up at finalization.
      */
 
-    if (firstContainerPtr == (Container *) NULL) {
+    if (tsdPtr->firstContainerPtr == (Container *) NULL) {
         Tcl_CreateExitHandler(CleanupContainerList, (ClientData) NULL);
     }
     
@@ -201,8 +208,8 @@ TkpUseWindow(interp, tkwin, string)
      * app. are in the same process.
      */
 
-    for (containerPtr = firstContainerPtr; containerPtr != NULL;
-	    containerPtr = containerPtr->nextPtr) {
+    for (containerPtr = tsdPtr->firstContainerPtr; 
+            containerPtr != NULL; containerPtr = containerPtr->nextPtr) {
 	if (containerPtr->parentHWnd == hwnd) {
 	    winPtr->flags |= TK_BOTH_HALVES;
 	    containerPtr->parentPtr->flags |= TK_BOTH_HALVES;
@@ -213,8 +220,8 @@ TkpUseWindow(interp, tkwin, string)
 	containerPtr = (Container *) ckalloc(sizeof(Container));
 	containerPtr->parentPtr = NULL;
 	containerPtr->parentHWnd = hwnd;
-	containerPtr->nextPtr = firstContainerPtr;
-	firstContainerPtr = containerPtr;
+	containerPtr->nextPtr = tsdPtr->firstContainerPtr;
+	tsdPtr->firstContainerPtr = containerPtr;
     }
 
     /*
@@ -258,13 +265,15 @@ TkpMakeContainer(tkwin)
 {
     TkWindow *winPtr = (TkWindow *) tkwin;
     Container *containerPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     /*
      * If this is the first container, register an exit handler so that
      * things will get cleaned up at finalization.
      */
 
-    if (firstContainerPtr == (Container *) NULL) {
+    if (tsdPtr->firstContainerPtr == (Container *) NULL) {
         Tcl_CreateExitHandler(CleanupContainerList, (ClientData) NULL);
     }
     
@@ -279,8 +288,8 @@ TkpMakeContainer(tkwin)
     containerPtr->parentHWnd = Tk_GetHWND(Tk_WindowId(tkwin));
     containerPtr->embeddedHWnd = NULL;
     containerPtr->embeddedPtr = NULL;
-    containerPtr->nextPtr = firstContainerPtr;
-    firstContainerPtr = containerPtr;
+    containerPtr->nextPtr = tsdPtr->firstContainerPtr;
+    tsdPtr->firstContainerPtr = containerPtr;
     winPtr->flags |= TK_CONTAINER;
 
     /*
@@ -358,12 +367,14 @@ TkWinEmbeddedEventProc(hwnd, message, wParam, lParam)
     LPARAM lParam;
 {
     Container *containerPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     /*
      * Find the Container structure associated with the parent window.
      */
 
-    for (containerPtr = firstContainerPtr;
+    for (containerPtr = tsdPtr->firstContainerPtr;
 	    containerPtr->parentHWnd != hwnd;
 	    containerPtr = containerPtr->nextPtr) {
 	if (containerPtr == NULL) {
@@ -386,7 +397,7 @@ TkWinEmbeddedEventProc(hwnd, message, wParam, lParam)
 
 	break;
       case TK_GEOMETRYREQ:
-	EmbedGeometryRequest(containerPtr, wParam, lParam);
+	EmbedGeometryRequest(containerPtr, (int) wParam, lParam);
 	break;
     }
     return 1;
@@ -508,8 +519,10 @@ TkpGetOtherWindow(winPtr)
 				 * embedded window. */
 {
     Container *containerPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
-    for (containerPtr = firstContainerPtr; containerPtr != NULL;
+    for (containerPtr = tsdPtr->firstContainerPtr; containerPtr != NULL;
 	    containerPtr = containerPtr->nextPtr) {
 	if (containerPtr->embeddedPtr == winPtr) {
 	    return containerPtr->parentPtr;
@@ -608,6 +621,8 @@ EmbedWindowDeleted(winPtr)
 				 * was deleted. */
 {
     Container *containerPtr, *prevPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     /*
      * Find the Container structure for this window work.  Delete the
@@ -616,7 +631,7 @@ EmbedWindowDeleted(winPtr)
      */
 
     prevPtr = NULL;
-    containerPtr = firstContainerPtr;
+    containerPtr = tsdPtr->firstContainerPtr;
     while (1) {
 	if (containerPtr->embeddedPtr == winPtr) {
 	    containerPtr->embeddedHWnd = NULL;
@@ -636,7 +651,7 @@ EmbedWindowDeleted(winPtr)
     if ((containerPtr->embeddedPtr == NULL)
 	    && (containerPtr->parentPtr == NULL)) {
 	if (prevPtr == NULL) {
-	    firstContainerPtr = containerPtr->nextPtr;
+	    tsdPtr->firstContainerPtr = containerPtr->nextPtr;
 	} else {
 	    prevPtr->nextPtr = containerPtr->nextPtr;
 	}
