@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkWinScrlbr.c,v 1.3 1999/04/16 01:51:53 stanton Exp $
+ * RCS: @(#) $Id: tkWinScrlbr.c,v 1.5.2.1 2001/04/04 07:57:18 hobbs Exp $
  */
 
 #include "tkWinInt.h"
@@ -177,11 +177,7 @@ UpdateScrollbar(scrollPtr)
     scrollInfo.nMin = 0;
     scrollInfo.nMax = MAX_SCROLL;
     thumbSize = (scrollPtr->info.lastFraction - scrollPtr->info.firstFraction);
-    if (tkpIsWin32s) {
-	scrollInfo.nPage = 0;
-    } else {
-	scrollInfo.nPage = ((UINT) (thumbSize * (double) MAX_SCROLL)) + 1;
-    } 
+    scrollInfo.nPage = ((UINT) (thumbSize * (double) MAX_SCROLL)) + 1;
     if (thumbSize < 1.0) {
 	scrollInfo.nPos = (int)
 	    ((scrollPtr->info.firstFraction / (1.0-thumbSize))
@@ -253,8 +249,13 @@ CreateProc(tkwin, parentWin, instanceData)
     }
 
     scrollPtr->lastVertical = scrollPtr->info.vertical;
+#ifdef _WIN64
+    scrollPtr->oldProc = (WNDPROC)SetWindowLongPtr(scrollPtr->hwnd,
+	    GWLP_WNDPROC, (LONG_PTR) ScrollbarProc);
+#else
     scrollPtr->oldProc = (WNDPROC)SetWindowLong(scrollPtr->hwnd, GWL_WNDPROC,
 	    (DWORD) ScrollbarProc);
+#endif
     window = Tk_AttachHWND(tkwin, scrollPtr->hwnd);
 
     UpdateScrollbar(scrollPtr);
@@ -299,7 +300,11 @@ TkpDisplayScrollbar(clientData)
     if (scrollPtr->lastVertical != scrollPtr->info.vertical) {
 	HWND hwnd = Tk_GetHWND(Tk_WindowId(tkwin));
 
+#ifdef _WIN64
+	SetWindowLongPtr(hwnd, GWLP_WNDPROC, (LONG_PTR) scrollPtr->oldProc);
+#else
 	SetWindowLong(hwnd, GWL_WNDPROC, (DWORD) scrollPtr->oldProc);
+#endif
 	DestroyWindow(hwnd);
 
 	CreateProc(tkwin, Tk_WindowId(Tk_Parent(tkwin)),
@@ -332,7 +337,11 @@ TkpDestroyScrollbar(scrollPtr)
     WinScrollbar *winScrollPtr = (WinScrollbar *)scrollPtr;
     HWND hwnd = winScrollPtr->hwnd;
     if (hwnd) {
+#ifdef _WIN64
+	SetWindowLongPtr(hwnd, GWLP_WNDPROC, (LONG_PTR) winScrollPtr->oldProc);
+#else
 	SetWindowLong(hwnd, GWL_WNDPROC, (DWORD) winScrollPtr->oldProc);
+#endif
 	if (winScrollPtr->winFlags & IN_MODAL_LOOP) {
 	    ((TkWindow *)scrollPtr->tkwin)->flags |= TK_DONT_DESTROY_WINDOW;
 	    SetParent(hwnd, NULL);
@@ -672,16 +681,18 @@ ModalLoopProc(tkwin, eventPtr)
     WinScrollbar *scrollPtr = (WinScrollbar *) winPtr->instanceData;
     int oldMode;
 
-    Tcl_Preserve((ClientData)scrollPtr);
-    scrollPtr->winFlags |= IN_MODAL_LOOP;
-    oldMode = Tcl_SetServiceMode(TCL_SERVICE_ALL);
-    TkWinResendEvent(scrollPtr->oldProc, scrollPtr->hwnd, eventPtr);
-    (void) Tcl_SetServiceMode(oldMode);
-    scrollPtr->winFlags &= ~IN_MODAL_LOOP;
-    if (scrollPtr->hwnd && scrollPtr->winFlags & ALREADY_DEAD) {
-	DestroyWindow(scrollPtr->hwnd);
+    if (scrollPtr->hwnd) {
+	Tcl_Preserve((ClientData)scrollPtr);
+	scrollPtr->winFlags |= IN_MODAL_LOOP;
+	oldMode = Tcl_SetServiceMode(TCL_SERVICE_ALL);
+	TkWinResendEvent(scrollPtr->oldProc, scrollPtr->hwnd, eventPtr);
+	(void) Tcl_SetServiceMode(oldMode);
+	scrollPtr->winFlags &= ~IN_MODAL_LOOP;
+	if (scrollPtr->hwnd && scrollPtr->winFlags & ALREADY_DEAD) {
+	    DestroyWindow(scrollPtr->hwnd);
+	}
+	Tcl_Release((ClientData)scrollPtr);
     }
-    Tcl_Release((ClientData)scrollPtr);
 }
 
 /*
