@@ -40,7 +40,11 @@ SOFTWARE.
 
 /* Resources */
 
+#ifdef _WIN64
+typedef __int64 XID;
+#else
 typedef unsigned long XID;
+#endif
 
 typedef XID Window;
 typedef XID Drawable;
